let currentDataset = [];
let currentTypes = [];

function initValuesFromPHPDump() {
    currentDataset = json_dump.articles; // json_dump is set using PHP
    currentTypes = json_dump.types;
    console.log(currentDataset);
    console.log(currentTypes);
    generateTable(currentDataset);
}



$(document).ready(function () {
    if (json_dump !== undefined) {
        initValuesFromPHPDump();
        generateTypeCheckboxes();
    }
    setEditFieldValues('', '', '', '', '', '', '');
});


function generateTable(dataset) {
    for (let i = 0; i < dataset.length; i++) {
        generateLine(dataset[i]);
    }
}

function generateLine(item) {
    $.selector_cache('#stockTable').append(
        '<tr id="row_' + item.code + '">' +
        '<td>' + item.name + '</td>' +
        '<td>' + item.description + '</td>' +
        '<td>' + item.quantity + '</td>' +
        '<td>' + item.price + '</td>' +
        '<td>' + item.code + '</td>' +
        '<td><span id="typeList_' + item.code + '"></span></td>' +
        '<td><a href="' + item.image + '" target="_blank"><img class="row-img" src="' + item.image + '"/></a></td>' +
        '<td>' +
        '<button class="btn btn-danger" id="delete_' + item.code + '" onclick="deleteItem(this)"><i class="fas fa-times"></i></button>' +
        '<button class="btn btn-primary" id="edit_' + item.code + '" onclick="editItem(this)"><i class="fas fa-edit"></i></button>' +
        '</td>' +
        '</tr>'
    );
    // Fill in the type cell
    for (let i = 0; i < item.type.length; i++) {
        let type = getTypeOfId(item.type[i]);
        $('#typeList_' + item.code).append(
            '<span class="mdi mdi-' + type.icon + '"></span>'
        );
    }
}

function generateTypeCheckboxes() {
    for (let i = 0; i < currentTypes.length; i++) {
        let id = 'typeCheck_' + currentTypes[i].id;
        $('#typeCheckboxesCell').append(
            '<div class="form-check">' +
            '<input type="checkbox" class="form-check-input" id="' + id + '">' +
            '<label class="form-check-label" for="' + id + '">' +
            '<span class="mdi mdi-' + currentTypes[i].icon + '"></span>' + currentTypes[i].name + '</label>' +
            '</div>'
        );
    }
}

function getTypeOfId(id) {
    let item = {};
    for (let i = 0; i < currentTypes.length; i++) {
        if (currentTypes[i].id === id) {
            item = currentTypes[i];
            break;
        }
    }
    return item;
}

function getItemOfCode(code) {
    let item = {};
    for (let i = 0; i < currentDataset.length; i++) {
        if (currentDataset[i].code === code) {
            item = currentDataset[i];
            break;
        }
    }
    return item;
}

function addNewItem() {
    if (isItemInputFilled()) {
        let item = {
            name: sanitizeString($.selector_cache('#nameInput').val()),
            description: sanitizeString($.selector_cache('#descriptionInput').val()),
            quantity: sanitizeNumber($.selector_cache('#quantityInput').val()),
            price: sanitizeNumber($.selector_cache('#priceInput').val()),
            code: $.selector_cache('#codeInput').val(),
            type: getTypesChecked(),
            image: $.selector_cache('#imageInput').val(),
        };
        if (isCodeAvailable(item.code)) {
            setEditFieldValues('', '', '', '', [], '', '');
            currentDataset.push(item);
            generateLine(item);
        }
    }
}

function editItem(elem) {
    if (isItemInputEmpty()) {
        let code = elem.id.replace('edit_', '');
        let item = getItemOfCode(code);
        setEditFieldValues(item.name, item.description, item.quantity, item.price, item.code, item.type, item.image);
        removeItemFromList(item); // Move the item in the edit fields
    }
}

function deleteItem(elem) {
    let code = elem.id.replace('delete_', '');
    let item = getItemOfCode(code);
    removeItemFromList(item);
}

function removeItemFromList(item) {
    currentDataset.splice(currentDataset.indexOf(item), 1);
    $('#row_' + item.code).remove();
}

function getTypesChecked() {
    let types = [];
    for (let i = 0; i < currentTypes.length; i++) {
        let id = 'typeCheck_' + currentTypes[i].id;
        if ($('#' + id).is(':checked')) {
            types.push(currentTypes[i].id);
        }
    }
    return types;
}

function setTypesChecked(types) {
    for (let i = 0; i < currentTypes.length; i++) {
        let id = 'typeCheck_' + currentTypes[i].id;
        $('#' + id).prop('checked', types.indexOf(currentTypes[i].id) !== -1);
    }
    return types;
}

function setEditFieldValues(name, description, quantity, price, code, type, image) {
    $.selector_cache('#nameInput').val(name);
    $.selector_cache('#descriptionInput').val(description);
    $.selector_cache('#quantityInput').val(quantity);
    $.selector_cache('#priceInput').val(price);
    $.selector_cache('#codeInput').val(code);
    $.selector_cache('#codeInput').val(code);
    setTypesChecked(type);
    $.selector_cache('#imageInput').val(image);
}

function isCodeAvailable (code) {
    let isAvailable = true;
    for (let i = 0; i < currentDataset.length; i++) {
        if (currentDataset[i].code === code) {
            isAvailable = false;
            break;
        }
    }
    return isAvailable;
}

function isItemInputEmpty() {
    return $.selector_cache('#nameInput').val() === '' &&
        $.selector_cache('#descriptionInput').val() === '' &&
        $.selector_cache('#quantityInput').val() === '' &&
        $.selector_cache('#priceInput').val() === '' &&
        $.selector_cache('#codeInput').val() === '' &&
        $.selector_cache('#imageInput').val() === '' &&
        getTypesChecked().length === 0;
}

function isItemInputFilled() {
    return $.selector_cache('#nameInput').val() !== '' &&
        $.selector_cache('#descriptionInput').val() !== '' &&
        $.selector_cache('#quantityInput').val() !== '' &&
        $.selector_cache('#priceInput').val() !== '' &&
        $.selector_cache('#codeInput').val() !== '' &&
        $.selector_cache('#imageInput').val() !== '' &&
        getTypesChecked().length > 0;
}



function saveDataset() {
    AjaxManager.saveData(currentTypes, currentDataset);
}

function sanitizeString(str) {
    return str.trim();
}

function sanitizeNumber(nbrStr) {
    return nbrStr.replace(/\s/g,'');
}

function scanArticle(code) {
    let data = {
        password: 'coucou',
        code : code,
    };
    $.ajax({
        type: "POST",
        url: "../ajax/scan_article.php",
        data: JSON.stringify(data),
        dataType: "json",
        contentType: "application/json; charset=utf-8",
        complete: function (data) {
            // alert(data.responseText);
            console.log(data.responseText);
        },
    });
}
